class PPyCException(Exception):
    exception_type: str = None
    message = None
    additional_info: dict = None
    code: str = None
    data: any = None

    def __init__(self, message=None, exception_type: str = None):
        super().__init__(message)
        self.exception_type = exception_type
        self.message = message

    def other_info(self, additional_info: dict = None, code: str = None):
        self.additional_info = additional_info
        self.code = code
        return self

    def add_additional_info(self, key: str, value):
        if not self.additional_info:
            self.additional_info = {}
        if key:
            self.additional_info[key] = value
        return self

    def add_data(self, data: any):
        self.data = data
        return self



import subprocess
import click


class Console:
    enable_log: bool = True
    log_starting: str = ">>"

    @staticmethod
    def print_log(text, color=None, bold=None, system_log: bool = False):
        if Console.enable_log or system_log:
            click.echo(click.style(text, fg=color, bold=bold))

    @staticmethod
    def green(text, bold=False, system_log: bool = False, enable_staring: bool = False):
        Console.print_log(Console.get_message_format(text, enable_staring=enable_staring), color="green", bold=bold, system_log=system_log)

    @staticmethod
    def blue(text, bold=False, system_log: bool = False, enable_staring: bool = False):
        Console.print_log(Console.get_message_format(text, enable_staring=enable_staring), color="blue", bold=bold, system_log=system_log)

    @staticmethod
    def red(text, bold=False, system_log: bool = False, enable_staring: bool = False):
        Console.print_log(Console.get_message_format(text, enable_staring=enable_staring), color="red", bold=bold, system_log=system_log)

    @staticmethod
    def yellow(text, bold=False, system_log: bool = False, enable_staring: bool = False):
        Console.print_log(Console.get_message_format(text, enable_staring=enable_staring), color="yellow", bold=bold, system_log=system_log)

    @staticmethod
    def magenta(text, bold=False, system_log: bool = False, enable_staring: bool = False):
        Console.print_log(Console.get_message_format(text, enable_staring=enable_staring), color="magenta", bold=bold, system_log=system_log)

    @staticmethod
    def cyan(text, bold=False, system_log: bool = False, enable_staring: bool = False):
        Console.print_log(Console.get_message_format(text, enable_staring=enable_staring), color="cyan", bold=bold, system_log=system_log)

    @staticmethod
    def get_message_format(text, enable_staring: bool = False):
        if not enable_staring:
            return text
        return f"{Console.log_starting} {str(text)}"

    @staticmethod
    def error(message, system_log: bool = False, enable_staring: bool = True):
        return Console.red(message, True, system_log=system_log, enable_staring=enable_staring)

    @staticmethod
    def success(message, system_log: bool = False, enable_staring: bool = True):
        return Console.green(message, True, system_log=system_log, enable_staring=enable_staring)

    @staticmethod
    def info(message, system_log: bool = False, enable_staring: bool = True):
        return Console.yellow(message, True, system_log=system_log, enable_staring=enable_staring)

    @staticmethod
    def log(message, system_log: bool = False, enable_staring: bool = True):
        return Console.blue(message, True, system_log=system_log, enable_staring=enable_staring)

    @staticmethod
    def run(command, home, env=None):
        response = subprocess.run(command, shell=True, cwd=home, env=env)
        return response



class DataUtil:

    @staticmethod
    def get_dict_value(data: dict, key: str, default=None):
        if not data or not key:
            return default
        elif key in data:
            return data[key]
        return default

    @staticmethod
    def get_object_fields(data_object: object):
        fields = []
        for field in dir(data_object):
            if not field.startswith('__'):
                fields.append(field)
        return fields

    @staticmethod
    def copy_object_to_object(source_object: object, dst_object: object):
        fields = DataUtil.get_object_fields(dst_object)
        if fields and source_object:
            for field in fields:
                if hasattr(source_object, field):
                    setattr(dst_object, field, getattr(source_object, field))
        return dst_object

    @staticmethod
    def copy_dict_to_object(dictionary: dict, data_object: object):
        if dictionary and data_object:
            for dict_key in dictionary:
                value = dictionary[dict_key]
                if hasattr(data_object, dict_key):
                    setattr(data_object, dict_key, value)
        return data_object



class ObjectHelper:

    @staticmethod
    def copy_config_property(source, destination):
        if source and destination:
            config_map = dir(source)
            for key in config_map:
                if key.isupper() and hasattr(destination, key):
                    setattr(destination, key, getattr(source, key))
        return destination



from ppy_common.ppyc_exception import PPyCException
from ppy_common.ppy_common_helper import PyCommon as PyCommon
import click
from ppy_common.ppy_object_helper import ObjectHelper
from ppy_common.ppyc_console_log import Console
from ppy_common.ppy_data_util import DataUtil
